package com.quintic.ble.qpp;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.text.Editable;
import android.text.InputFilter;
import android.text.Spanned;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.quintic.libqpp.QppApi;
import com.quintic.libqpp.iQppCallback;

public class QppActivity extends Activity {
	protected static final String TAG = QppActivity.class.getSimpleName();
	private BluetoothManager mBluetoothManager=null;
	private static BluetoothAdapter mBluetoothAdapter=null;
	public static BluetoothGatt mBluetoothGatt=null;
	
	public static final String EXTRAS_DEVICE_NAME = "deviceName";
	public static final String EXTRAS_DEVICE_ADDRESS = "deviceAddress";
	
	private boolean dataRecvFlag=false;
	private String deviceName;
	private String deviceAddress;

	/** connection state */
	private boolean mConnected = false; 
	private boolean mConnected_last = false;
	/** scan all Service ?*/
	private boolean isInitialize = false;
	private static final int MAX_DATA_SIZE=20; 
	/// public 
	private TextView textDeviceName;
	private TextView textDeviceAddress;
	private TextView textConnectionStatus;
	
	/// qpp start
	protected static String uuidQppService = "0000fee9-0000-1000-8000-00805f9b34fb";
	protected static String uuidQppCharWrite = "d44bc439-abfd-45a2-b575-925416129600";

	protected boolean qppConfirmFlag;
	
	/// receive data
	private TextView textQppNotify;
	private TextView textQppDataRate;

	private long qppSumDataReceived = 0;   /// summary of data received.
	private long qppRecvDataTime=0;
	/// send 
	private EditText editSend;
	private Button btnQppTextSend;
	
	/// repeat start
	private CheckBox checkboxRepeat;	
	private boolean qppSendDataState = false;
	
	private TextView labelRepeatCounter;
	private TextView textRepeatCounter;
	
	private EditText edtdisp;
	private TextView disnumTextView;
	private TextView txnumTextView;
	
	private EditText intervalTime;
		
	
	private long qppRepeatCounter = 0;
	/// repeat end
	
	private long rxnums=0;
	private long txnums=0;
	private boolean dispen=false;
	private boolean isReceHex = false;
	private boolean isSendHex = false;
	
	private boolean isrepeat=false;
	
	private CheckBox disCheckBox;
	
	private byte qppDataSend[] = {0x00,'1', '2','3', '4','5','6', '7',
            '8','9', 'a', 'b', 'c','d','e', 'f', 0x10,0x11,0x12,0x13}; 

	private int tempNum=500;
	private byte[]sendDat;
	private boolean menuDelay = false;
	
	/* 延时线程 */
	private Handler mHandler;
	private boolean mChanging = false;
    // can change after 1 seconds.
    private static final long CHANGE_PERIOD = 1000;
    
    //input data limited
    private int editStart;//光标开始位置  
    private int editEnd;//光标结束位置 
    
	private boolean initialize() {
	    // For API level 18 and above, get a reference to BluetoothAdapter through BluetoothManager.
	    if (mBluetoothManager == null) {
		mBluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
		if (mBluetoothManager == null) {
		    Log.e(TAG, "Unable to initialize BluetoothManager.");
		    return false;
		}
	    }	        
	    mBluetoothAdapter = mBluetoothManager.getAdapter();
	    if (mBluetoothAdapter == null) {
		Log.e(TAG, "Unable to obtain a BluetoothAdapter.");
		return false;
	    }		
	    return true;
	}
/**
 * get the number of the send editText prepared to send
 * @return the sizes of send
 */
	private int getEditTextSendLength() {
		  byte[] dat = null;
		   if (!isSendHex) {
		    	dat = editSend.getText().toString().getBytes();
		    } else {
		    	dat = hexToChar(editSend.getText().toString().getBytes());
		    }
		   return dat.length;
	}
	
	private byte[] hexToChar (final byte[]dat) {
		byte[] new_byte= new byte[dat.length];
	
		int step = 0, index = 0;
		char ch = 0, ch1 = 0;
		for (int n = 0; n < dat.length; n++) {
			switch (step) {
			
			case 0:
				if ((ch = (char) dat[n]) != ' ') {
					step = 1;
				}
				break;
		   //
			case 1:
				if ( (ch1 = (char)dat[n]) == ' ') {
					new_byte[index++] = (byte) (ch-'0');
				} else {
					ch =  (char) (ch  <='9'?  (ch -'0') << 4 : ((ch -'A') + 10) << 4);
					ch1 = (char) (ch1 <='9'?  (ch1-'0') << 0 : ((ch1-'A') + 10) << 0);
					new_byte[index++] = (byte) (ch | ch1);
				}
				step = 0;
				break;
			}
			
		}
		
		byte[] sendbyte= new byte[index];
		System.arraycopy(new_byte, 0, sendbyte, 0, index); 
		new_byte = null;
		
		return sendbyte;
	}
	
	private Handler handlersend = new Handler( );
	final Runnable runnableSend = new Runnable( ) {
		private void QppSendNextData() {
		    qppDataSend[0]++;
		    byte[] dat = null;
		    if (!isSendHex) {
		    	dat = editSend.getText().toString().getBytes();
		    } else {
		    	dat = hexToChar(editSend.getText().toString().getBytes());
		    }
		
		    if(!QppApi.qppSendData(mBluetoothGatt, dat))//qppDataSend))
		    {
			Log.e(TAG,"send data failed");
			return;
		    }
		
			txnums += dat.length;
			txnumTextView.setText("TX bytes: "+String.valueOf(txnums));
        	
		    qppRepeatCounter++;
		    setRepeatCounter("   "+qppRepeatCounter+"Times");
//		    try {
//				Thread.sleep(tempNum);
//			} catch (InterruptedException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
		}
		public void run ( ) {
		    QppSendNextData();
        	}
	};	
	
	private Handler TimeDelay = new Handler( );
	final Runnable disableMenu = new Runnable() {

		@Override
		public void run() {
			// TODO Auto-generated method stub
			try {
				Thread.sleep(1000);
				
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}finally{
				menuDelay = false;				
			}
			
		}
		
	};
	
	private void clearHandler(Handler handler,Runnable runner)
	{
	    if(handler!=null){
		handler.removeCallbacks(runner);
		handler=null;
	    }
	}
	private final BluetoothGattCallback mGattCallback = new BluetoothGattCallback(){
		@Override
		public void onConnectionStateChange(BluetoothGatt gatt, int status,int newState) {
			Log.i(TAG, "onConnectionStateChange : "+status+"  newState : "+newState);
			if(newState == BluetoothProfile.STATE_CONNECTED){			
			    mBluetoothGatt.discoverServices();			    
			    mConnected = true;			    			  			    			    
			}else if(newState== BluetoothProfile.STATE_DISCONNECTED){
			    
			    mConnected = false;				
			    clearHandler(handlerQppDataRate,runnableQppDataRate);
			    clearHandler(handlersend,runnableSend);
			    dataRecvFlag=false;
			    if(qppSendDataState){
				setBtnSendState("Send");
				qppSendDataState=false;
			    }
			    close();
			}
	        if(!mChanging) { //delay for moments
	        	invalidateOptionsMenu();  			
	        }
		}	

		@Override
		public void onServicesDiscovered(BluetoothGatt gatt, int status) {		    
		    if(QppApi.qppEnable(mBluetoothGatt, uuidQppService, uuidQppCharWrite)){	
		    	isInitialize = true;
		    	setConnectState(R.string.qpp_support);
		    }else{
		    	isInitialize = false;
		    	List<BluetoothGattService> lBluetoothGattServices;
		    	lBluetoothGattServices=gatt.getServices();
		    	if(lBluetoothGattServices==null)
		    	{
		    		setConnectState(R.string.qpp_not_support);
		    	}
		    	else {
//		    		if(lBluetoothGattServices.size()>0)
//		    			setQppData(new String());  		
		    		//setConnectState(lBluetoothGattServices.get(0).getUuid().clockSequence());
				}
		    }
		}			
		
		@Override
		public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {       			          	
        	    QppApi.updateValueForNotification(gatt, characteristic);        	            	    
		}		
		
		@Override
		public void onDescriptorWrite(BluetoothGatt gatt,BluetoothGattDescriptor descriptor, int status) {
			//super.onDescriptorWrite(gatt, descriptor, status);
		    Log.w(TAG,"onDescriptorWrite");
		    QppApi.setQppNextNotify(gatt, true);	
		}
		
		@Override
		public void onCharacteristicWrite(BluetoothGatt gatt,BluetoothGattCharacteristic characteristic, int status) {
		    //super.onCharacteristicWrite(gatt, characteristic, status);
		    if(status == BluetoothGatt.GATT_SUCCESS && qppSendDataState)
		    {
			/*This is a workaround,20140819,xiesc: it paused with unknown reason on android 4.4.3 
			 */
			if(handlersend!=null && runnableSend!=null)
			     handlersend.postDelayed(runnableSend,tempNum);
			    //handlersend.post(runnableSend);
		    }else
		    {
			Log.e(TAG,"Send failed!!!!");
		    }
		} 			
	};	
	public boolean connect(final String address) {
	    if (mBluetoothAdapter == null || address == null) {
		Log.w("Qn Dbg", "BluetoothAdapter not initialized or unspecified address.");
		return false;
	    }
	    final BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
	    if (device == null) {
		Log.w(TAG, "Device not found.  Unable to connect.");
		return false;
	    }
	        //setting the autoConnect parameter to false.
	    mBluetoothGatt = device.connectGatt(this, false, mGattCallback);
	        
	    Log.d(TAG, "Trying to create a new connection. Gatt: " + mBluetoothGatt);
	    return true;
	}
	public void disconnect() {
	    if (mBluetoothAdapter == null || mBluetoothGatt == null) {
	            Log.w("Qn Dbg", "BluetoothAdapter not initialized");
	            return;
	    }       	        
	    mBluetoothGatt.disconnect();
	}
	    
	public void close() {
	    if (mBluetoothGatt != null) {
		    mBluetoothGatt.close();
		    mBluetoothGatt = null;
	    }
	}
	
	
	final Handler handlerQppDataRate = new Handler( );
	final Runnable runnableQppDataRate = new Runnable( ) {
        public void run ( ) {
           qppRecvDataTime++;
           textQppDataRate.setText(" "+qppSumDataReceived/qppRecvDataTime+" Bps");  
             
           dataRecvFlag=false;
        }
	};
   	 
    private void setConnectState(final int stat) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
        	textConnectionStatus.setText(stat);
            }
        });       
    }    
      
    private void setQppNotify(final String errStr) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
        	//textQppNotify.setText(errStr);
            }
        });        
    }
    
    private String bytesToHex(final byte[] dataBytes) {
    	char temp;
    	String str = "";
    	for (int n=0; n<dataBytes.length; n++) {
    		temp = (char) ((dataBytes[n] & 0xf0) >> 4);
    		str += (char)( temp >= 10? 'A'+(temp-10):'0'+temp);
    		temp = (char) ((dataBytes[n] & 0x0f) >> 0);
    		str += (char)( temp >= 10? 'A'+(temp-10):'0'+temp);
    		str +=  ' ';
    	}
    	return str;
    }
    
    private void setQppData(final byte[] dataBytes) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
            	
            	if(dispen){
            		if(!isReceHex) { // show as char
            			String errStr = new String(dataBytes);
            			edtdisp.append(errStr);
            		} else { // show as hex
            			edtdisp.append(bytesToHex(dataBytes));
            		}
            	}
            	rxnums+=dataBytes.length;
            	disnumTextView.setText("RX bytes: "+String.valueOf(rxnums));
            }
        });        
    } 
    private void setRepeatCounter(final String errStr) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
        	textRepeatCounter.setText(errStr);
            }
        });        
    }
    private void setBtnSendState(final String str) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
        	btnQppTextSend.setText(str);
            }
        });        
    }        
    

    private InputFilter[] getHexNormalFilter() {
    	return new InputFilter[] {
    			new InputFilter.LengthFilter(MAX_DATA_SIZE*4),
    			new InputFilter.AllCaps(),
    			new InputFilter() { 
    			    public CharSequence filter(CharSequence source, int start, int end, Spanned dest, int dstart,
    			 int dend) { 
    			    	
    			            for (int i = start; i < end; i++) { 
    			                    if ( !(source.charAt(i) >= 'A' && source.charAt(i) <= 'F' ) &&
    			                         !(source.charAt(i) >= 'a' && source.charAt(i) <= 'f' ) &&
    			                         !(source.charAt(i) >= '0' && source.charAt(i) <= '9' ) &&
    			                         !Character.toString(source.charAt(i)) .equals(" ")){ 
    			                    	 Toast.makeText(getApplicationContext(), getResources().getString(R.string.toast_input_error), Toast.LENGTH_LONG).show();
    			                         return ""; 
    			                    } 
    			            } 
    			            return null; 
    			    }  }};
    }
    @Override
    protected void onCreate(Bundle savedInstanceState) {
	super.onCreate(savedInstanceState);
	setContentView( R.layout.acitvity_appmy);
	
	getActionBar().setDisplayHomeAsUpEnabled(true);		
	mHandler = new Handler();
	
	textDeviceName = (TextView)findViewById(R.id.text_device_name);
	textDeviceAddress = (TextView)findViewById(R.id.text_device_address);
	textConnectionStatus = (TextView)findViewById(R.id.text_connection_state);		
	editSend = (EditText)findViewById(R.id.edit_send);
	editSend.setFilters(new InputFilter[]{new InputFilter.LengthFilter(MAX_DATA_SIZE + 1)});
	
	editSend.addTextChangedListener(new TextWatcher() {
		
		@Override
		public void onTextChanged(CharSequence s, int start, int before, int count) {
			// TODO Auto-generated method stub
		}
		
		@Override
		public void beforeTextChanged(CharSequence s, int start, int count,
				int after) {
			// TODO Auto-generated method stub
			
		}
		
		@Override
		public void afterTextChanged(Editable s) {
			// TODO Auto-generated method stub
	         editStart = editSend.getSelectionStart();
	         editEnd = editSend.getSelectionEnd();
	         String str = editSend.getText().toString();
	        if (editEnd >= 3 && isSendHex) {
	        	if ( str.charAt(editEnd - 1) != ' ' &&
	        		 str.charAt(editEnd - 2) != ' ' &&
	        		 str.charAt(editEnd - 3) != ' ') {
	        		 s.delete(editStart - 1, editEnd);
		             int tempSelection = editStart;
		             editSend.setText(s);
		             editSend.setSelection(tempSelection);
		             
	        	}
	        }
	        if (getEditTextSendLength() > 20) {
	            Toast.makeText(getApplicationContext(), getResources().getString(R.string.toast_msg), Toast.LENGTH_LONG).show();
	            s.delete(editStart - 1, editEnd);
	            int tempSelection = editStart;
	            editSend.setText(s);
	            editSend.setSelection(tempSelection);
	        } 
	        
		}
	});
	btnQppTextSend = (Button)findViewById(R.id.btn_qpp_text_send);	
//	checkboxRepeat = (CheckBox)findViewById(R.id.cb_repeat);
	labelRepeatCounter = (TextView)findViewById(R.id.label_repeat_counter);
	textRepeatCounter = (TextView)findViewById(R.id.text_repeat_counter);
	//textQppNotify = (TextView)findViewById(R.id.text_qpp_notify);
	textQppDataRate = (TextView)findViewById(R.id.text_qpp_data_rate);
	intervalTime = (EditText)findViewById(R.id.text_interval);
	
	deviceName = getIntent().getExtras().getString(EXTRAS_DEVICE_NAME);
	deviceAddress =getIntent().getExtras().getString(EXTRAS_DEVICE_ADDRESS);		
		
	textDeviceName.setText(deviceName);
	textDeviceAddress.setText(deviceAddress);
	
	edtdisp = (EditText)findViewById(R.id.editText1);
	edtdisp.setText("");
	edtdisp.setScrollBarStyle(View.SCROLLBARS_INSIDE_INSET);
	disnumTextView=(TextView)findViewById(R.id.checkedTextView1);
	RelativeLayout relativeLayoutclear=(RelativeLayout)findViewById(R.id.relativeLayout_rece_clear);
	relativeLayoutclear.setOnClickListener(new OnClickListener() {
		
		@Override
		public void onClick(View arg0) {
			// TODO Auto-generated method stub
			edtdisp.setText("");
			rxnums=0;
			disnumTextView.setText("RX bytes: 0");
		}
	});
	
	txnumTextView=(TextView)findViewById(R.id.checkedTextViewTx);
	RelativeLayout relativeLayoutsendclear=(RelativeLayout)findViewById(R.id.relativeLayout_send_clear);
	relativeLayoutsendclear.setOnClickListener(new OnClickListener() {
		
		@Override
		public void onClick(View arg0) {
			// TODO Auto-generated method stub
			txnums=0;
			qppRepeatCounter = 0;
			txnumTextView.setText("TX bytes: 0");
		}
	});
	
	RelativeLayout relativeLayoutdis=(RelativeLayout)findViewById(R.id.relativeLayout_rece_view);
	relativeLayoutdis.setOnClickListener(new OnClickListener() {
		
		@Override
		public void onClick(View arg0) {
			// TODO Auto-generated method stub
			Drawable dr = null; 
		
			if(!dispen){
				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_pressed);
				dispen=true;
			}
			else {
				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_normal);
				dispen=false;
			}
			((RelativeLayout)arg0).setBackground(dr);
		}
	});

	RelativeLayout relativeLayoutrecehex=(RelativeLayout)findViewById(R.id.relativeLayout_rece_hex);
	relativeLayoutrecehex.setOnClickListener(new OnClickListener() {
		
		@Override
		public void onClick(View arg0) {
			// TODO Auto-generated method stub
			Drawable dr = null; 
		
			if(!isReceHex){
				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_pressed);
				isReceHex=true;
				
			}
			else {
				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_normal);
				isReceHex=false;
			}
			((RelativeLayout)arg0).setBackground(dr);
		}
	});
	
	RelativeLayout relativeLayoutsendhex=(RelativeLayout)findViewById(R.id.relativeLayout_send_hex);
	relativeLayoutsendhex.setOnClickListener(new OnClickListener() {
		
		@Override
		public void onClick(View arg0) {
			// TODO Auto-generated method stub
			Drawable dr = null; 
		
			if(!isSendHex){
				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_pressed);
				isSendHex=true;
				editSend.setFilters(getHexNormalFilter()); 
				editStart = 0;
				editEnd   = 0;
				String str = bytesToHex(editSend.getText().toString().getBytes());
				editSend.setText(str);
			}
			else {
				editSend.setFilters(new InputFilter[]{new InputFilter.LengthFilter(MAX_DATA_SIZE)});
				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_normal);
				isSendHex=false;
				editSend.setText(new String(hexToChar(editSend.getText().toString().getBytes())));
				editSend.setFilters(new InputFilter[]{new InputFilter.LengthFilter(MAX_DATA_SIZE + 1)});
			}
			((RelativeLayout)arg0).setBackground(dr);
		}
	});
	
	if (!initialize()) {
	    Log.e(TAG, "Unable to initialize Bluetooth");
	    finish();
	}	
		
	QppApi.setCallback(new iQppCallback(){
			
	    @Override
	    public void onQppReceiveData(BluetoothGatt mBluetoothGatt, String qppUUIDForNotifyChar, byte[] qppData) {
		if(!dataRecvFlag){
		    handlerQppDataRate.postDelayed(runnableQppDataRate,1000);
		    dataRecvFlag=true;
		} 
  		qppSumDataReceived=qppSumDataReceived+qppData.length;
  		//setQppNotify((byte)qppData[0]+" "+(byte)qppData[1]);
  		mBluetoothGatt.readRemoteRssi();
  		//setQppNotify((byte)qppData[0]+" "+(byte)qppData[1]+" data:"+new String(qppData));
  		setQppData(qppData);  		
	    }
	}); 
	
	TextWatcher textwatcher = new TextWatcher(){

		@Override
		public void afterTextChanged(Editable s) {
			// TODO Auto-generated method stub
			 tempNum = Integer.parseInt(s.toString());
			 if(tempNum<30){
				 tempNum = 30;
			 }
		}

		@Override
		public void beforeTextChanged(CharSequence s, int start, int count,
				int after) {
			// TODO Auto-generated method stub
			
		}

		@Override
		public void onTextChanged(CharSequence s, int start, int before,
				int count) {
			// TODO Auto-generated method stub
			
			
		}	
	};
	intervalTime.addTextChangedListener(textwatcher );
		
		/**
		 * start to send qpp package OR RESET...
		 */
        btnQppTextSend.setOnClickListener(new OnClickListener(){
            @Override
            public void onClick(View arg0) {
            if(!mConnected || !isInitialize){
            	Toast.makeText(QppActivity.this, "Please connect device first and ensure your device support Qpp service!", Toast.LENGTH_SHORT).show();
            	return ;
            }

            if(isrepeat){	
				if(!qppSendDataState){
				    qppRepeatCounter = 0;
				    qppSendDataState = true;		    
				    btnQppTextSend.setText("Stop");
//				    sendDat=editSend.getText().toString().getBytes();
				    handlersend.post(runnableSend);
				}else{
				    qppSendDataState = false;
				    btnQppTextSend.setText("Send");
				}
            } else{
            	if(!qppSendDataState){
//            		sendDat=editSend.getText().toString().getBytes();
            		handlersend.post(runnableSend);
            	}else{
            		qppSendDataState = false;
				    btnQppTextSend.setText("Send");
            	}
            }							
        }

            private byte[] packageDataSend() { 
	    int data_sent_lenth = editSend.getText().length();
	    String strInput;
				
	    strInput =  editSend.getText().toString();		
	    if((data_sent_lenth % 2) == 1){
		strInput = "0"+strInput;
	    }
	    return	HexBytesUtils.hexStr2Bytes(strInput);
	}
	});
        

    	RelativeLayout relativeLayoutrepeat=(RelativeLayout)findViewById(R.id.relativeLayout_send_repeat);
    	relativeLayoutrepeat.setOnClickListener(new OnClickListener() {
    		
    		@Override
    		public void onClick(View arg0) {
    			// TODO Auto-generated method stub
    			Drawable dr = null; 
    			labelRepeatCounter.setVisibility(View.VISIBLE);
    			textRepeatCounter.setVisibility(View.VISIBLE);
    			if(!isrepeat){
    				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_pressed);
    				isrepeat=true;
    			}
    			else {
    				dr = getResources().getDrawable(R.drawable.ic_preference_small_single_normal);
    				isrepeat=false;
    			}
    			((RelativeLayout)arg0).setBackground(dr);
    		}
    	});

    	
	}		

	@Override	
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.qpp, menu);
		
		
		if (mChanging) {
			menu.findItem(R.id.menu_refresh).setActionView(R.layout.actionbar_indeterminate_progress);
			if (mConnected_last) {
				menu.findItem(R.id.menu_connect).setVisible(false);
				menu.findItem(R.id.menu_disconnect).setVisible(false);
				menu.findItem(R.id.menu_connecting).setVisible(false);
				menu.findItem(R.id.menu_disconnecting).setVisible(true);
			} else {
				menu.findItem(R.id.menu_connect).setVisible(false);
				menu.findItem(R.id.menu_disconnect).setVisible(false);
				menu.findItem(R.id.menu_connecting).setVisible(true);
				menu.findItem(R.id.menu_disconnecting).setVisible(false);
			}
		} else {
			menu.findItem(R.id.menu_refresh).setActionView(null);
			if (mConnected) {
				menu.findItem(R.id.menu_connect).setVisible(false);
				menu.findItem(R.id.menu_disconnect).setVisible(true);
				menu.findItem(R.id.menu_connecting).setVisible(false);
				menu.findItem(R.id.menu_disconnecting).setVisible(false);
			} else {
				menu.findItem(R.id.menu_connect).setVisible(true);
				menu.findItem(R.id.menu_disconnect).setVisible(false);
				menu.findItem(R.id.menu_connecting).setVisible(false);
				menu.findItem(R.id.menu_disconnecting).setVisible(false);
			}
		}
		return true;
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
//		if(menuDelay==false) {
//			menuDelay=true;
//			TimeDelay.post(disableMenu);
			
			switch (item.getItemId()) {
			case R.id.menu_connect:
				changeConnectState(false);
				return true;
			case R.id.menu_disconnect:
				changeConnectState(true);
				return true;
			case android.R.id.home:
				onBackPressed();
				return true;
//			}			
		}
		
		return super.onOptionsItemSelected(item);
	}	
	
    /*
     * when call this function, the flag will be setted true ofter CHANGE_PEPRIOD 
     * to avoid change connect or disconnect too fast   
     * */
	private void changeConnectState(boolean b) {
	
		if (!mChanging) {
	        if (b) {/* Disconect */
	        	disconnect();
	        	
	        } else { /* Conect */
	        	connect(deviceAddress);
	        }
	      
	        mHandler.postDelayed(new Runnable() {
	            @Override
	            public void run() {
	            	mChanging = false;
	                invalidateOptionsMenu();
	            }
	        }, CHANGE_PERIOD);
	       mChanging = true;
	       mConnected_last = mConnected;
	       invalidateOptionsMenu();
	       
    	}
	}
	@Override
	protected void onResume() {
		super.onResume();

		if(!mConnected)
		{
		    textConnectionStatus.setText(R.string.qpp_not_support);
		    if(!mChanging) { //delay for moments
	        	invalidateOptionsMenu();  			
	        } 
	        connect(deviceAddress);
	    }            
	}	  

	@Override
	protected void onPause() {
		super.onPause();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		clearHandler(handlerQppDataRate,runnableQppDataRate);
		clearHandler(handlersend,runnableSend);		
		close();
	}
}
